/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: MatrixUnaryFunctions.h,v 1.8 2004/01/27 18:06:47 opetzold Exp $
 */

#ifndef TVMET_MATRIX_UNARY_FUNCTIONS_H
#define TVMET_MATRIX_UNARY_FUNCTIONS_H

namespace tvmet {


/*
 * unary_function(Matrix<T, Rows, Cols>)
 */
#define TVMET_IMPLEMENT_MACRO(FUNC, FUNCTIONAL)				\
template<class T, std::size_t Rows, std::size_t Cols>			\
inline									\
XprMatrix<								\
  XprUnOp<								\
    FUNCTIONAL<T>, 							\
    MatrixConstReference<T, Rows, Cols>					\
  >, 									\
  Rows, Cols				      			      	\
>									\
FUNC(const Matrix<T, Rows, Cols>& rhs) {				\
  typedef XprUnOp<							\
    FUNCTIONAL<T>, 							\
    MatrixConstReference<T, Rows, Cols>					\
  > 							expr_type;	\
  return XprMatrix<expr_type, Rows, Cols>(expr_type(rhs.const_ref()));	\
}

TVMET_IMPLEMENT_MACRO(abs, Fcnl_abs)
TVMET_IMPLEMENT_MACRO(cbrt, Fcnl_cbrt)
TVMET_IMPLEMENT_MACRO(ceil, Fcnl_ceil)
TVMET_IMPLEMENT_MACRO(floor, Fcnl_floor)
TVMET_IMPLEMENT_MACRO(rint, Fcnl_rint)
TVMET_IMPLEMENT_MACRO(sin, Fcnl_sin)
TVMET_IMPLEMENT_MACRO(cos, Fcnl_cos)
TVMET_IMPLEMENT_MACRO(tan, Fcnl_tan)
TVMET_IMPLEMENT_MACRO(sinh, Fcnl_sinh)
TVMET_IMPLEMENT_MACRO(cosh, Fcnl_cosh)
TVMET_IMPLEMENT_MACRO(tanh, Fcnl_tanh)
TVMET_IMPLEMENT_MACRO(asin, Fcnl_asin)
TVMET_IMPLEMENT_MACRO(acos, Fcnl_acos)
TVMET_IMPLEMENT_MACRO(atan, Fcnl_atan)
TVMET_IMPLEMENT_MACRO(exp, Fcnl_exp)
TVMET_IMPLEMENT_MACRO(log, Fcnl_log)
TVMET_IMPLEMENT_MACRO(log10, Fcnl_log10)
TVMET_IMPLEMENT_MACRO(sqrt, Fcnl_sqrt)

#if defined(TVMET_HAVE_IEEE_MATH)
TVMET_IMPLEMENT_MACRO(asinh, Fcnl_asinh)
TVMET_IMPLEMENT_MACRO(acosh, Fcnl_acosh)
TVMET_IMPLEMENT_MACRO(atanh, Fcnl_atanh)
TVMET_IMPLEMENT_MACRO(expm1, Fcnl_expm1)
TVMET_IMPLEMENT_MACRO(log1p, Fcnl_log1p)
TVMET_IMPLEMENT_MACRO(erf, Fcnl_erf)
TVMET_IMPLEMENT_MACRO(erfc, Fcnl_erfc)
TVMET_IMPLEMENT_MACRO(j0, Fcnl_j0)
TVMET_IMPLEMENT_MACRO(j1, Fcnl_j1)
TVMET_IMPLEMENT_MACRO(y0, Fcnl_y0)
TVMET_IMPLEMENT_MACRO(y1, Fcnl_y1)
TVMET_IMPLEMENT_MACRO(lgamma, Fcnl_lgamma)
/** \todo isnan etc. - default return is only an int! */
#if !defined(TVMET_NO_IEEE_MATH_ISNAN)
TVMET_IMPLEMENT_MACRO(isnan, Fcnl_isnan)
#endif
#if !defined(TVMET_NO_IEEE_MATH_ISINF)
TVMET_IMPLEMENT_MACRO(isinf, Fcnl_isinf)
#endif
TVMET_IMPLEMENT_MACRO(finite, Fcnl_finite)
#endif // defined(TVMET_HAVE_IEEE_MATH)

#undef TVMET_IMPLEMENT_MACRO


/*
 * unary_function(Matrix<std::complex<T>, Rows, Cols>)
 */
#if defined(TVMET_HAVE_COMPLEX)
#define TVMET_IMPLEMENT_MACRO(FUNC, FUNCTIONAL)				\
template<class T, std::size_t Rows, std::size_t Cols>			\
inline									\
XprMatrix<								\
  XprUnOp<								\
    FUNCTIONAL< std::complex<T> >,					\
    MatrixConstReference<std::complex<T>, Rows, Cols>			\
  >,									\
  Rows, Cols								\
>									\
FUNC(const Matrix<std::complex<T>, Rows, Cols>& rhs) {			\
  typedef XprUnOp<							\
    FUNCTIONAL< std::complex<T> >,					\
    MatrixConstReference<std::complex<T>, Rows, Cols>			\
  > 							expr_type;	\
  return XprMatrix<expr_type, Rows, Cols>(expr_type(rhs.const_ref()));	\
}

TVMET_IMPLEMENT_MACRO(real, Fcnl_real)
TVMET_IMPLEMENT_MACRO(imag, Fcnl_imag)
TVMET_IMPLEMENT_MACRO(arg, Fcnl_arg)
TVMET_IMPLEMENT_MACRO(norm, Fcnl_norm)
TVMET_IMPLEMENT_MACRO(conj, Fcnl_conj)

#undef TVMET_IMPLEMENT_MACRO

#endif // defined(TVMET_HAVE_COMPLEX)


} // namespace tvmet

#endif // TVMET_MATRIX_UNARY_FUNCTIONS_H

// Local Variables:
// mode:C++
// End:
