/*
 * Tiny Vector Matrix Library
 * Dense Vector Matrix Libary of Tiny size using Expression Templates
 *
 * Copyright (C) 2001 - 2003 Olaf Petzold <opetzold@users.sourceforge.net>
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2.1 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public
 * License along with this library; if not, write to the Free Software
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 * $Id: UnaryFunctionals.h,v 1.14 2003/12/05 19:08:41 opetzold Exp $
 */

#ifndef TVMET_UNARY_FUNCTIONAL_H
#define TVMET_UNARY_FUNCTIONAL_H

namespace tvmet {


/**
 * \class Fcnl_LogicalNot	UnaryFunctionals.h "tvmet/UnaryFunctionals.h"
 * \brief unary functional for logical not.
 */
template <class T>
struct Fcnl_LogicalNot : public UnaryFunctional {
  typedef bool						value_type;
  static inline value_type apply_on(T rhs) {
    return !rhs;
  }
  static void print_xpr(std::ostream& os, std::size_t l=0) {
    os << IndentLevel(l) << "Fcnl_LogicalNot<T="
       << typeid(T).name() << ">,"
       << std::endl;
  }
};


/** \class Fcnl_BitwiseNot	UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_Negate		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
#define TVMET_IMPLEMENT_MACRO(OPNAME, OP)                   		\
template <class T>	                                 		\
struct OPNAME : public UnaryFunctional {                		\
  typedef T						value_type;	\
  static inline value_type apply_on(T rhs) {		        	\
    return OP rhs;                                         		\
  }                                                         		\
  static void print_xpr(std::ostream& os, std::size_t l=0) {    	\
    os << IndentLevel(l) << #OPNAME << "<T="	 			\
       << typeid(T).name() << ">,"                         		\
       << std::endl;                          				\
  }     								\
};

TVMET_IMPLEMENT_MACRO(Fcnl_BitwiseNot, ~)
TVMET_IMPLEMENT_MACRO(Fcnl_Negate, -)
#undef TVMET_IMPLEMENT_MACRO


/** \class Fcnl_abs		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_cbrt		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_ceil		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_floor		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_rint		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_sin		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_cos		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_tan		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_sinh		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_cosh		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_tanh		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_asin		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_acos		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_atan		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_exp		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_log		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_log10		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_sqrt		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
#define TVMET_IMPLEMENT_MACRO(FUNCNAME, MATHFUNC)             		\
template <class T>	                                 		\
struct FUNCNAME : public UnaryFunctional {                		\
  typedef T						value_type;	\
  static inline value_type apply_on(T rhs) {		        	\
    return MATHFUNC(rhs);               				\
  }                                                         		\
  static void print_xpr(std::ostream& os, std::size_t l=0) {    		\
    os << IndentLevel(l) << #FUNCNAME << "<T="	 			\
       << typeid(T).name() << ">,"                         		\
       << std::endl;                          				\
  }     								\
};

TVMET_IMPLEMENT_MACRO(Fcnl_abs, std::abs) // specialized later, see below
TVMET_IMPLEMENT_MACRO(Fcnl_cbrt, std::cbrt)
TVMET_IMPLEMENT_MACRO(Fcnl_ceil, std::ceil)
TVMET_IMPLEMENT_MACRO(Fcnl_floor, std::floor)
TVMET_IMPLEMENT_MACRO(Fcnl_rint, std::rint)
TVMET_IMPLEMENT_MACRO(Fcnl_sin, std::sin)
TVMET_IMPLEMENT_MACRO(Fcnl_cos, std::cos)
TVMET_IMPLEMENT_MACRO(Fcnl_tan, std::tan)
TVMET_IMPLEMENT_MACRO(Fcnl_sinh, std::sinh)
TVMET_IMPLEMENT_MACRO(Fcnl_cosh, std::cosh)
TVMET_IMPLEMENT_MACRO(Fcnl_tanh, std::tanh)
TVMET_IMPLEMENT_MACRO(Fcnl_asin, std::asin)
TVMET_IMPLEMENT_MACRO(Fcnl_acos, std::acos)
TVMET_IMPLEMENT_MACRO(Fcnl_atan, std::atan)
TVMET_IMPLEMENT_MACRO(Fcnl_exp, std::exp)
TVMET_IMPLEMENT_MACRO(Fcnl_log, std::log)
TVMET_IMPLEMENT_MACRO(Fcnl_log10, std::log10)
TVMET_IMPLEMENT_MACRO(Fcnl_sqrt, std::sqrt)

#if defined(TVMET_HAVE_IEEE_MATH)
/** \class Fcnl_asinh		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_acosh		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_atanh		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_expm1		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_log1p		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_erf		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_erfc		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_j0		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_j1		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_y0		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_y1		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_lgamma		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
TVMET_IMPLEMENT_MACRO(Fcnl_asinh, ::asinh)
TVMET_IMPLEMENT_MACRO(Fcnl_acosh, ::acosh)
TVMET_IMPLEMENT_MACRO(Fcnl_atanh, ::atanh)
TVMET_IMPLEMENT_MACRO(Fcnl_expm1, ::expm1)
TVMET_IMPLEMENT_MACRO(Fcnl_log1p, ::log1p)
TVMET_IMPLEMENT_MACRO(Fcnl_erf, ::erf)
TVMET_IMPLEMENT_MACRO(Fcnl_erfc, ::erfc)
TVMET_IMPLEMENT_MACRO(Fcnl_j0, ::j0)
TVMET_IMPLEMENT_MACRO(Fcnl_j1, ::j1)
TVMET_IMPLEMENT_MACRO(Fcnl_y0, ::y0)
TVMET_IMPLEMENT_MACRO(Fcnl_y1, ::y1)
TVMET_IMPLEMENT_MACRO(Fcnl_lgamma, ::lgamma)
#endif // defined(TVMET_HAVE_IEEE_MATH)

#undef TVMET_IMPLEMENT_MACRO


/** \class Fcnl_abs<long int>		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_abs<long long int>	UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_abs<float>		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_abs<double>		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_abs<long double> 	UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
#define TVMET_IMPLEMENT_MACRO(FUNCNAME, MATHFUNC, POD)    		\
template <>	                                 			\
struct FUNCNAME< POD > : public UnaryFunctional {           		\
  typedef POD						value_type; 	\
  static inline value_type apply_on(POD rhs) {	        		\
    return MATHFUNC(rhs);               				\
  }                                                         		\
  static void print_xpr(std::ostream& os, std::size_t l=0) {    		\
    os << IndentLevel(l) << #FUNCNAME << "<T="	 			\
       << typeid(POD).name() << ">,"                         		\
       << std::endl;                          				\
  }     								\
};

TVMET_IMPLEMENT_MACRO(Fcnl_abs, std::labs, long int)

#if defined(TVMET_HAVE_LONG_LONG)
TVMET_IMPLEMENT_MACRO(Fcnl_abs, std::labs, long long int)
#endif // defined(TVMET_HAVE_LONG_LONG)

TVMET_IMPLEMENT_MACRO(Fcnl_abs, std::fabs, float)
TVMET_IMPLEMENT_MACRO(Fcnl_abs, std::fabs, double)

#if defined(TVMET_HAVE_LONG_DOUBLE)
TVMET_IMPLEMENT_MACRO(Fcnl_abs, std::fabs, long double)
#endif // defined(TVMET_HAVE_LONG_DOUBLE)

#undef TVMET_IMPLEMENT_MACRO

/*
 * complex support
 */

#if defined(TVMET_HAVE_COMPLEX)
/**
 * \class Fcnl_abs< std::complex<T> > UnaryFunctionals.h "tvmet/UnaryFunctionals.h"
 */
template <class T>
struct Fcnl_abs< std::complex<T> > : public UnaryFunctional {
  typedef T						value_type;
  static inline value_type apply_on(const std::complex<T>& rhs) {
    return std::abs(rhs);
  }
  static void print_xpr(std::ostream& os, std::size_t l=0) {
    os << IndentLevel(l) << "Fcnl_abs<T="
       << typeid(std::complex<T>).name() << ">,"
       << std::endl;
  }
};


/**
 * \class Fcnl_conj< std::complex<T> > UnaryFunctionals.h "tvmet/UnaryFunctionals.h"
 * \brief %Functional for conj.
 */
template <class T> struct Fcnl_conj : public UnaryFunctional { };


/** \class Fcnl_conj< std::complex<T> > UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
template <class T>
struct Fcnl_conj< std::complex<T> > : public UnaryFunctional {
  typedef std::complex<T>                               value_type;
  static inline value_type apply_on(const std::complex<T>& rhs) {
    return std::conj(rhs);
  }
  static void print_xpr(std::ostream& os, std::size_t l=0) {
    os << IndentLevel(l) << "Fcnl_conj<T="
       << typeid(std::complex<T>).name() << ">,"
       << std::endl;
  }
};


/** \class Fcnl_real< std::complex<T> > UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_imag< std::complex<T> > UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_arg< std::complex<T> > UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_norm< std::complex<T> > UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
#define TVMET_IMPLEMENT_MACRO(FUNCNAME, MATHFUNC)             		\
template <class T> struct FUNCNAME : public UnaryFunctional { };        \
template <class T>	                                 		\
struct FUNCNAME< std::complex<T> > : public UnaryFunctional {  		\
  typedef T						value_type;	\
  static inline value_type apply_on(const std::complex<T>& rhs) {     	\
    return MATHFUNC(rhs);               				\
  }                                                         		\
  static void print_xpr(std::ostream& os, std::size_t l=0) {    		\
    os << IndentLevel(l) << #FUNCNAME << "<T="	 			\
       << typeid(std::complex<T>).name() << ">,"              		\
       << std::endl;                          				\
  }     								\
};

TVMET_IMPLEMENT_MACRO(Fcnl_real, std::real)
TVMET_IMPLEMENT_MACRO(Fcnl_imag, std::imag)
TVMET_IMPLEMENT_MACRO(Fcnl_arg, std::arg)
TVMET_IMPLEMENT_MACRO(Fcnl_norm, std::norm)

#undef TVMET_IMPLEMENT_MACRO

#endif // defined(TVMET_HAVE_COMPLEX)


/** \class Fcnl_isnan		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_isinf		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
/** \class Fcnl_finite		UnaryFunctionals.h "tvmet/UnaryFunctionals.h" */
#define TVMET_IMPLEMENT_MACRO(FUNCNAME, MATHFUNC, POD)    		\
template <class T>                                 			\
struct FUNCNAME : public UnaryFunctional {           			\
  typedef T						value_type;	\
  static inline POD apply_on(T rhs) {		        		\
    return MATHFUNC(rhs);               				\
  }                                                         		\
  static void print_xpr(std::ostream& os, std::size_t l=0) {    		\
    os << IndentLevel(l) << #FUNCNAME << "<T="	 			\
       << typeid(POD).name() << ">,"                         		\
       << std::endl;                          				\
  }     								\
};

TVMET_IMPLEMENT_MACRO(Fcnl_isnan, ::isnan, int)
TVMET_IMPLEMENT_MACRO(Fcnl_isinf, ::isinf, int)
TVMET_IMPLEMENT_MACRO(Fcnl_finite, ::finite, int)

#undef TVMET_IMPLEMENT_MACRO


} // namespace tvmet

#endif // TVMET_UNARY_FUNCTIONAL_H

// Local Variables:
// mode:C++
// End:
